package main

import (
  "bufio"
  "fmt"
  "os"
  "strconv"
  "strings"
)

type studente struct {
  nominativo string
  votiEsami []int
}

func avg(votiEsami []int) (float64, bool) {
  if len(votiEsami) == 0 {
    return 0, false
  }
  sum := 0
  for _, voto := range votiEsami {
    if voto != 32 {
      sum += voto
    } else {
      sum += 30
    }
  }
  return float64(sum) / float64(len(votiEsami)), true
}

func trentToCentd(media float64) float64 {
  return media * 11 / 3
}

func parseLine(line string) studente {
  var stud studente

  parts := strings.Split(line, ",")
  stud.nominativo = parts[0]    // TODO: e se il nominativo è vuoto?
  stud.votiEsami = []int{}
  for i := 1; i < len(parts); i++ {
    voto, err := strconv.Atoi(parts[i])
    if err != nil || voto < 18 || (voto > 30 && voto != 32){
      fmt.Fprintf(os.Stderr, "Errore nello studente %s, voto %s --- ", stud.nominativo, parts[i])
      fmt.Fprintln(os.Stderr, "Ignorato!")
    } else {
      stud.votiEsami = append(stud.votiEsami, voto)
    }
  }
  return stud
}

func readFile(fileName string) (ris []studente) {
  var stud studente

  f, err := os.Open(fileName)
  defer f.Close()
  if err != nil {
    fmt.Fprintln(os.Stderr, "Errore di apertura del file! ", err)
    os.Exit(1)
  }
  scanner := bufio.NewScanner(f)
  for scanner.Scan() {
    line := scanner.Text()  // Leggo una riga dal fileName
    stud = parseLine(line)
    ris = append(ris, stud)
  }
  return
}

func writeFile(fileName string, dati []studente) {
  f, err := os.Create(fileName)
  defer f.Close()
  if err != nil {
    fmt.Fprintln(os.Stderr, "Errore di apertura del file! ", err)
    os.Exit(1)
  }
  for _, stud := range dati {
    media, ok := avg(stud.votiEsami)
    if ok {
      media110 := trentToCentd(media)
      fmt.Fprintf(f, "%-40s\t%8.2f\n", stud.nominativo, media110)
    }
  }
}

func main() {
  inputFileName := os.Args[1]
  outputFileName := os.Args[2]
  dati := readFile(inputFileName)
  writeFile(outputFileName, dati)
}
